unit TeeGraph;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, Menus, ExtCtrls, TeeProcs, TeEngine, Chart, Series,
  MiscRoutines;

type
  TGraphForm = class(TForm)
    Chart: TChart;
    TeeGraphMainMenu: TMainMenu;
    TeeEditMenu: TMenuItem;
    TeeFormatItem: TMenuItem;
    TeeExportItem: TMenuItem;
    Series: TLineSeries;
    SaveDialog: TSaveDialog;
    CopyChartMenu: TMenuItem;
    BitmapItem: TMenuItem;
    MetafileItem: TMenuItem;
    ILNormal: TImageList;
    Zoom11Item: TMenuItem;
    Points: TPointSeries;
    IncMenu: TMenuItem;
    XIncItem: TMenuItem;
    YIncItem: TMenuItem;
    procedure TeeExportItemClick(Sender: TObject);
    procedure MetafileItemClick(Sender: TObject);
    procedure BitmapItemClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure Zoom11ItemClick(Sender: TObject);
    procedure TeeFormatItemClick(Sender: TObject);
    procedure XIncItemClick(Sender: TObject);
    procedure YIncItemClick(Sender: TObject);
  private
  public
    NotifyTeeClose : TNotifyEvent;
    procedure SetupChanged;  
  end;

implementation

uses Setup;

{$R *.DFM}

procedure TGraphForm.TeeExportItemClick(Sender: TObject);
var
  DriveDir,Name,Ext : string;
  OldColor : integer;
begin
  if(not SaveDialog.Execute) then exit;
  SplitPath(SaveDialog.FileName,DriveDir,Name,Ext);
  StripLeadingAndTrailing(DriveDir);
  StripLeadingAndTrailing(Name);
  StripLeadingAndTrailing(Ext);
  if(Ext='') then
  begin
    case SaveDialog.FilterIndex of
    1: Ext:='wmf';
    2: Ext:='emf';
    end;
  end;
  SaveDialog.FileName:=MakePath(DriveDir,Name,Ext);
  OldColor:=Chart.Color;
  Chart.Color:=AppSetup.TeeFlags.CopyColor;
  case SaveDialog.FilterIndex of
  1: Chart.SaveToMetafile(SaveDialog.FileName);
  2: Chart.SaveToMetafileEnh(SaveDialog.FileName);
  end;
  Chart.Color:=OldColor;
end;

procedure TGraphForm.MetafileItemClick(Sender: TObject);
var
  OldColor : integer;
begin
  OldColor:=Chart.Color;
  Chart.Color:=AppSetup.TeeFlags.CopyColor;
  Chart.CopyToClipboardMetafile(False);
  Chart.Color:=OldColor;
end;

procedure TGraphForm.BitmapItemClick(Sender: TObject);
var
  OldColor : integer;
begin
  OldColor:=Chart.Color;
  Chart.Color:=AppSetup.TeeFlags.CopyColor;
  Chart.CopyToClipboardBitmap;
  Chart.Color:=OldColor;
end;

procedure TGraphForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if(@NotifyTeeClose<>nil) then
    NotifyTeeClose(Self);
  Action:=caFree;
end;

procedure TGraphForm.FormCreate(Sender: TObject);
begin
  NotifyTeeClose:=nil;
end;

procedure TGraphForm.Zoom11ItemClick(Sender: TObject);
begin
  Chart.UndoZoom;
end;

procedure TGraphForm.TeeFormatItemClick(Sender: TObject);
begin
  OptionsForm.PCOptions.ActivePage:=OptionsForm.TeeSheet;
  OptionsForm.ShowModal;
end;

procedure TGraphForm.SetupChanged;
begin
  Chart.Title.Text.Clear;
  Chart.Title.Text.Add(string(AppSetup.TeeFlags.Legend));
  Chart.Color:=AppSetup.TeeFlags.Color;
  Chart.LeftAxis.Axis.Color:=AppSetup.TeeFlags.AxisColor;
  Chart.BottomAxis.Axis.Color:=AppSetup.TeeFlags.AxisColor;
  Chart.LeftAxis.Grid.Visible:=AppSetup.TeeFlags.GridVisible;
  Chart.BottomAxis.Grid.Visible:=AppSetup.TeeFlags.GridVisible;
  Chart.LeftAxis.Grid.Color:=AppSetup.TeeFlags.GridColor;
  Chart.BottomAxis.Grid.Color:=AppSetup.TeeFlags.GridColor;
  Chart.LeftAxis.Grid.Style:=AppSetup.TeeFlags.GridStyle;
  Chart.BottomAxis.Grid.Style:=AppSetup.TeeFlags.GridStyle;
  Series.LinePen.Color:=AppSetup.TeeFlags.SeriesColor;
end;

procedure TGraphForm.XIncItemClick(Sender: TObject);
var
  newinc : double;
begin
  if(DlgGetDouble(newinc,FloatToStr(Chart.BottomAxis.Increment),
    '   ')) then
    Chart.BottomAxis.Increment:=newinc;
end;

procedure TGraphForm.YIncItemClick(Sender: TObject);
var
  newinc : double;
begin
  if(DlgGetDouble(newinc,FloatToStr(Chart.LeftAxis.Increment),
    '   ')) then
    Chart.LeftAxis.Increment:=newinc;
end;

end.

